#include "ScriptComponent.h"
#include "../utils/Logging.h"
#include "../utils/ExceptionHandler.h"
#include "../utils/GameObjectsList.h"

#include "../Entitys/Object.h"
#include "../Components/SpriteComponent.h"
#include "../Components/CameraComponent.h"
#include "../Components/LightComponent.h"
#include "../Components/TextComponent.h"
#include "../Components/TilemapComponent.h"

#include <lua.hpp>
#include <memory>
#include <cstring>

static bool luaDebugEnabled = false;

struct LuaObjectWrapper {
    Object* obj;
};
#define LUA_OBJECT_MT "LuaObjectMeta"

struct LuaVector2 {
    float x, y;
};
#define LUA_VECTOR2_MT "LuaVector2Meta"

ScriptComponent::ScriptComponent(Object* owner) : Component(owner), L(nullptr) {}
ScriptComponent::~ScriptComponent() { if (L) lua_close(L); }

void ScriptComponent::SetScriptPath(const std::string& path) {
    scriptPath = path;
    ReloadScript();
}
const std::string& ScriptComponent::GetScriptPath() const { return scriptPath; }

// Logging bindings
static int Lua_LogInfo(lua_State* L) {
    Logger::LogInfo("[Lua] %s", lua_tostring(L, 1));
    return 0;
}
static int Lua_LogError(lua_State* L) {
    Logger::LogError("[Lua] %s", lua_tostring(L, 1));
    return 0;
}
static int Lua_LogDebug(lua_State* L) {
    Logger::LogDebug("[Lua] %s", lua_tostring(L, 1));
    return 0;
}
static int Lua_DebugLua(lua_State* L) {
    luaDebugEnabled = lua_toboolean(L, 1);
    Logger::LogInfo("[Lua] DebugLua = %s", luaDebugEnabled ? "true" : "false");
    return 0;
}

// Component resolver
static Component* GetComponentByName(Object* obj, const std::string& type) {
    if (type == "SpriteComponent")   return obj->GetComponent<SpriteComponent>().get();
    if (type == "CameraComponent")   return obj->GetComponent<CameraComponent>().get();
    if (type == "LightComponent")    return obj->GetComponent<LightComponent>().get();
    if (type == "TilemapComponent")  return obj->GetComponent<TilemapComponent>().get();
    if (type == "TextComponent")     return obj->GetComponent<TextComponent>().get();
    if (type == "ScriptComponent")   return obj->GetComponent<ScriptComponent>().get();
    return nullptr;
}

// Object:GetComponent("Type")
static int Lua_Object_GetComponent(lua_State* L) {
    auto* wrapper = (LuaObjectWrapper*)luaL_checkudata(L, 1, LUA_OBJECT_MT);
    const char* type = luaL_checkstring(L, 2);

    Component* comp = GetComponentByName(wrapper->obj, type);
    if (comp) lua_pushlightuserdata(L, comp);
    else lua_pushnil(L);
    return 1;
}

// Object:GetPosition()
static int Lua_Object_GetPosition(lua_State* L) {
    auto* wrapper = (LuaObjectWrapper*)luaL_checkudata(L, 1, LUA_OBJECT_MT);
    glm::vec2 pos = wrapper->obj->GetLocalPosition();

    LuaVector2* vec = (LuaVector2*)lua_newuserdata(L, sizeof(LuaVector2));
    vec->x = pos.x;
    vec->y = pos.y;
    luaL_getmetatable(L, LUA_VECTOR2_MT);
    lua_setmetatable(L, -2);
    return 1;
}

// Object:SetPosition(Vector2)
static int Lua_Object_SetPosition(lua_State* L) {
    auto* wrapper = (LuaObjectWrapper*)luaL_checkudata(L, 1, LUA_OBJECT_MT);
    auto* vec = (LuaVector2*)luaL_checkudata(L, 2, LUA_VECTOR2_MT);
    wrapper->obj->SetLocalPosition({ vec->x, vec->y });
    return 0;
}

// __index for Object
static int Lua_Object_Index(lua_State* L) {
    const char* key = luaL_checkstring(L, 2);
    lua_getfield(L, lua_upvalueindex(1), key);
    return 1;
}

void RegisterObjectType(lua_State* L) {
    luaL_newmetatable(L, LUA_OBJECT_MT);

    lua_newtable(L); // method table
    lua_pushcfunction(L, Lua_Object_GetComponent); lua_setfield(L, -2, "GetComponent");
    lua_pushcfunction(L, Lua_Object_GetPosition); lua_setfield(L, -2, "GetPosition");
    lua_pushcfunction(L, Lua_Object_SetPosition); lua_setfield(L, -2, "SetPosition");

    lua_pushcclosure(L, Lua_Object_Index, 1);
    lua_setfield(L, -2, "__index");

    lua_pop(L, 1);
}

static void PushObject(lua_State* L, Object* obj) {
    auto* wrapper = (LuaObjectWrapper*)lua_newuserdata(L, sizeof(LuaObjectWrapper));
    wrapper->obj = obj;
    luaL_getmetatable(L, LUA_OBJECT_MT);
    lua_setmetatable(L, -2);
}

// Engine.GetObjectByTag(name)
static int Lua_GetObjectByTag(lua_State* L) {
    const char* name = luaL_checkstring(L, 1);
    for (const auto& obj : objects) {
        auto found = FindByTagRecursive(obj, name);
        if (found) {
            PushObject(L, found.get());
            return 1;
        }
    }
    lua_pushnil(L);
    return 1;
}

// Vector2(x, y)
static int Lua_Vector2_New(lua_State* L) {
    LuaVector2* vec = (LuaVector2*)lua_newuserdata(L, sizeof(LuaVector2));
    vec->x = (float)luaL_optnumber(L, 1, 0);
    vec->y = (float)luaL_optnumber(L, 2, 0);
    luaL_getmetatable(L, LUA_VECTOR2_MT);
    lua_setmetatable(L, -2);
    return 1;
}

static int Lua_Vector2_Index(lua_State* L) {
    auto* vec = (LuaVector2*)luaL_checkudata(L, 1, LUA_VECTOR2_MT);
    const char* key = luaL_checkstring(L, 2);
    if (strcmp(key, "x") == 0) lua_pushnumber(L, vec->x);
    else if (strcmp(key, "y") == 0) lua_pushnumber(L, vec->y);
    else lua_pushnil(L);
    return 1;
}
static int Lua_Vector2_NewIndex(lua_State* L) {
    auto* vec = (LuaVector2*)luaL_checkudata(L, 1, LUA_VECTOR2_MT);
    const char* key = luaL_checkstring(L, 2);
    float value = (float)luaL_checknumber(L, 3);
    if (strcmp(key, "x") == 0) vec->x = value;
    else if (strcmp(key, "y") == 0) vec->y = value;
    return 0;
}

void RegisterVector2Type(lua_State* L) {
    luaL_newmetatable(L, LUA_VECTOR2_MT);
    lua_pushcfunction(L, Lua_Vector2_Index); lua_setfield(L, -2, "__index");
    lua_pushcfunction(L, Lua_Vector2_NewIndex); lua_setfield(L, -2, "__newindex");
    lua_pop(L, 1);
    lua_pushcfunction(L, Lua_Vector2_New);
    lua_setglobal(L, "Vector2");
}

void ScriptComponent::RegisterEngineBindings() {
    lua_newtable(L);

    lua_pushcfunction(L, Lua_LogInfo); lua_setfield(L, -2, "LogInfo");
    lua_pushcfunction(L, Lua_LogError); lua_setfield(L, -2, "LogError");
    lua_pushcfunction(L, Lua_LogDebug); lua_setfield(L, -2, "LogDebug");
    lua_pushcfunction(L, Lua_GetObjectByTag); lua_setfield(L, -2, "GetObjectByTag");
    lua_pushcfunction(L, Lua_DebugLua); lua_setfield(L, -2, "DebugLua");

    lua_setglobal(L, "Engine");
}

void ScriptComponent::ReloadScript() {
    if (scriptPath.empty()) return;

    if (L) lua_close(L);
    L = luaL_newstate();
    luaL_openlibs(L);

    RegisterObjectType(L);
    RegisterVector2Type(L);
    RegisterEngineBindings();

    if (luaL_dofile(L, scriptPath.c_str())) {
        Logger::LogError("[Lua] %s", lua_tostring(L, -1));
        RecoverableError("Failed to load Lua script: " + scriptPath, Create::Exceptions::ComponentLoad).Handle();
        return;
    }

    lua_getglobal(L, "OnInit");
    if (lua_isfunction(L, -1)) {
        if (lua_pcall(L, 0, 0, 0) != LUA_OK) {
            Logger::LogError("[Lua] %s", lua_tostring(L, -1));
            RecoverableError("OnInit failed: " + scriptPath, Create::Exceptions::ComponentLoad).Handle();
        }
    } else {
        lua_pop(L, 1);
    }
}

void ScriptComponent::OnUpdate(float dt) {
    if (!L) return;
    lua_getglobal(L, "OnUpdate");
    if (lua_isfunction(L, -1)) {
        lua_pushnumber(L, dt);
        if (lua_pcall(L, 1, 0, 0) != LUA_OK) {
            Logger::LogError("[Lua] %s", lua_tostring(L, -1));
            RecoverableError("OnUpdate failed in: " + scriptPath, Create::Exceptions::ComponentLoad).Handle();
        }
    } else {
        lua_pop(L, 1);
    }
}

void ScriptComponent::Save(YAML::Emitter& out) const {
    out << YAML::BeginMap;
    out << YAML::Key << "type" << YAML::Value << "ScriptComponent";
    out << YAML::Key << "scriptPath" << YAML::Value << scriptPath;
    out << YAML::EndMap;
}

void ScriptComponent::Load(const YAML::Node& node) {
    try {
        if (!node["scriptPath"]) {
            RecoverableError("Missing 'scriptPath' in ScriptComponent", Create::Exceptions::MissingField).Handle();
            return;
        }

        scriptPath = node["scriptPath"].as<std::string>();
        ReloadScript();
    } catch (const YAML::Exception& e) {
        RecoverableError("YAML error in ScriptComponent::Load: " + std::string(e.what()), Create::Exceptions::ComponentLoad).Handle();
    }
}
