#include "Engine.h"
#include "Entitys/Object.h"
#include "Renderer.h"
#include "components/SpriteComponent.h"
#include "components/CameraComponent.h"
#include "components/LightComponent.h"
#include "components/TilemapComponent.h"
#include "components/ScriptComponent.h"

#include "utils/FileDialog.h"
#include "utils/Logging.h"

#include "utils/EngineConfig.h"
#include "utils/GameObjectsList.h"

#include <GL/glew.h>
#include <GLFW/glfw3.h>
#include <imgui.h>
#include <imgui_impl_glfw.h>
#include <imgui_impl_opengl3.h>

#include <functional>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <vector>
#include <memory>
#include <filesystem>
#include <iostream>
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <commdlg.h>
#include <openssl/sha.h>

static std::shared_ptr<Object> selected = nullptr;
static bool playing = false;
static std::vector<std::shared_ptr<Object>> pendingDeletion;

static glm::vec2 cameraPos = {0, 0};
static float cameraZoom = 1.0f;
static ImVec2 lastMousePos = {};

static const std::string tempScenePath = "__tmp_scene.yaml";

static float g_fps = 0.0f;

GLFWwindow *window = nullptr;

Engine::Engine()
{
    Init();
}

Engine::~Engine()
{
    Shutdown();
}

void Engine::ShowDebugOverlay(float deltaTime)
{
    static std::vector<float> fpsHistory;
    static const int maxHistory = 100;
    static float timer = 0;
    float fps = 1.0f / deltaTime;
    g_fps = fps;
    if (fpsHistory.size() >= maxHistory)
        fpsHistory.erase(fpsHistory.begin());
    if (timer >= 0.05)
    {
        timer = 0;
        fpsHistory.push_back(fps);
    }
    timer += deltaTime;

    ImGui::Begin("Performance Info", nullptr,
                 ImGuiWindowFlags_NoDecoration);

    ImGui::TextColored(ImVec4(0.3f, 1.0f, 0.3f, 1.0f), "Performance");
    ImGui::Separator();
    ImGui::Text("FPS:        %.1f", fps);
    ImGui::Text("Delta Time: %.4f s", deltaTime);
    ImGui::Text("Frame Time: %.2f ms", deltaTime * 1000.0f);
    ImGui::PlotLines("##FPS", fpsHistory.data(), fpsHistory.size(), 0, "FPS History", 0.0f, 144.0f, ImVec2(-1, 50));

    ImGui::Spacing();
    ImGui::TextColored(ImVec4(0.4f, 0.7f, 1.0f, 1.0f), "Renderer");
    ImGui::Separator();
    ImGui::Text("Draw Calls: %d", Renderer::GetDrawCallCount());
    ImGui::Text("Scene Lights: %d/%d", Renderer::GetLightsCount(), g_engineConfig.gl_maxLight);

    ImGui::Text("Reserved Draws: %d", m_Reserved_draws);

    ImGui::Text("Camera Zoom: %.2f", cameraZoom);
    ImGui::Text("Camera Pos:  (%.1f, %.1f)", cameraPos.x, cameraPos.y);

    ImGui::End();
}

void Engine::Init()
{
    glfwInit();
    glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3);
    glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 3);
    glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);

    window = glfwCreateWindow(1280, 720, "Create Editor", nullptr, nullptr);
    glfwMakeContextCurrent(window);
    glfwSwapInterval(0); // No VSync
    glewInit();

    IMGUI_CHECKVERSION();
    ImGui::CreateContext();
    ImGuiIO &io = ImGui::GetIO();
    io.ConfigFlags |= ImGuiConfigFlags_DockingEnable;
    ImGui::StyleColorsDark();
    ImGui_ImplGlfw_InitForOpenGL(window, true);
    ImGui_ImplOpenGL3_Init((std::string("#version ") + g_engineConfig.gl_version).c_str());
    m_Reserved_draws = 0;

    Renderer::Init();

    auto obj = std::make_shared<Object>("Hello, Create");

    objects.push_back(obj);
    selected = obj;
    Logger::LogInfo("Initialized Engine");
}
void DrawInspectorUI(std::shared_ptr<Object> selected)
{
    ImGui::Begin("Inspector");

    if (!selected)
    {
        ImGui::TextDisabled("No object selected.");
        ImGui::End();
        return;
    }

    // --- keep track of the name buffer ---
    static char nameBuffer[128] = "";
    static std::shared_ptr<Object> lastSelected = nullptr;
    if (selected != lastSelected)
    {
        strncpy(nameBuffer, selected->GetName().c_str(), sizeof(nameBuffer));
        nameBuffer[sizeof(nameBuffer) - 1] = '\0';
        lastSelected = selected;
    }

    // --- General ---
    ImGui::TextUnformatted("General");
    ImGui::Separator();

    // Name edit
    if (ImGui::InputText("Name", nameBuffer, sizeof(nameBuffer)))
        selected->SetName(nameBuffer);

    // Visibility toggle
    bool visible = selected->GetVisable();
    if (ImGui::Checkbox("Visible", &visible))
        selected->SetVisable(visible);

    // --- Transform ---
    ImGui::TextUnformatted("Transform");
    ImGui::Separator();

    glm::vec2 pos = selected->GetLocalPosition();
    if (ImGui::DragFloat2("Position", &pos.x, 0.1f))
        selected->SetLocalPosition(pos);

    int layer = selected->layer;
    if (ImGui::InputInt("Layer", &layer))
        selected->layer = layer;

    ImGui::Text("ID: %d", selected->uid.id);
    ImGui::Text("UID: %s", selected->uid.uuid.c_str());

    ImGui::Spacing();
    ImGui::SeparatorText("Add Component");

    static const char *componentTypes[] = {
        "SpriteComponent",
        "CameraComponent",
        "LightComponent",
        "ScriptComponent",
        "TilemapComponent"};

    static int selectedIndex = -1;

    if (ImGui::BeginCombo("##ComponentCombo", selectedIndex >= 0 ? componentTypes[selectedIndex] : "Select Component"))
    {
        for (int i = 0; i < IM_ARRAYSIZE(componentTypes); ++i)
        {
            bool isSelected = (selectedIndex == i);
            if (ImGui::Selectable(componentTypes[i], isSelected))
                selectedIndex = i;
            if (isSelected)
                ImGui::SetItemDefaultFocus();
        }
        ImGui::EndCombo();
    }

    ImGui::SameLine();
    if (ImGui::Button("Add") && selectedIndex != -1 && selected)
    {
        const std::string type = componentTypes[selectedIndex];

        if (type == "SpriteComponent" && !selected->GetComponent<SpriteComponent>())
            selected->AddComponent<SpriteComponent>();
        else if (type == "CameraComponent" && !selected->GetComponent<CameraComponent>())
            selected->AddComponent<CameraComponent>();
        else if (type == "LightComponent" && !selected->GetComponent<LightComponent>())
            selected->AddComponent<LightComponent>();
        else if (type == "ScriptComponent" && !selected->GetComponent<ScriptComponent>())
            selected->AddComponent<ScriptComponent>();
        else if (type == "TilemapComponent" && !selected->GetComponent<TilemapComponent>())
            selected->AddComponent<TilemapComponent>();
    }

    if (auto sprite = selected->GetComponent<SpriteComponent>())
    {
        ImGui::SeparatorText("Sprite Component");

        const char *renderTypes[] = {"Unlit", "Lit"};
        SpriteComponent::RenderType currentType = sprite->GetRenderType();
        int currentTypeIndex = static_cast<int>(currentType);

        if (ImGui::Combo("Render Type", &currentTypeIndex, renderTypes, IM_ARRAYSIZE(renderTypes)))
        {
            sprite->SetRenderType(static_cast<SpriteComponent::RenderType>(currentTypeIndex));
        }

        std::string tex = sprite->GetTexturePath();
        ImGui::Text("Texture: %s", tex.c_str());
        if (ImGui::Button("Load Texture"))
        {
            auto path = OpenFileDialog(FileDialogType::Images);
            if (!path.empty())
                sprite->SetTexture(path);
        }

        std::string norm = sprite->GetNormalMapPath();
        ImGui::Text("Normal Map: %s", norm.c_str());
        if (ImGui::Button("Load Normal Map"))
        {
            auto path = OpenFileDialog(FileDialogType::Images);
            if (!path.empty())
                sprite->SetNormalMap(path);
        }

        glm::vec2 size = sprite->GetSize();
        ImGui::Text("Size: %.0f x %.0f", size.x, size.y);

        if (ImGui::Button("Remove SpriteComponent"))
            selected->RemoveComponent<SpriteComponent>();
    }

    if (auto cam = selected->GetComponent<CameraComponent>())
    {
        ImGui::SeparatorText("Camera Component");

        float fov = cam->GetFOV();
        float aspect = cam->GetAspect();
        float zoom = cam->GetZoom();

        if (ImGui::DragFloat("FOV", &fov, 0.1f, 1.0f, 179.0f))
            cam->SetFOV(fov);
        if (ImGui::DragFloat("Aspect", &aspect, 0.01f, 0.1f, 5.0f))
            cam->SetAspect(aspect);
        if (ImGui::DragFloat("Zoom", &zoom, 0.1f, 0.1f, 10.0f))
            cam->SetZoom(zoom);

        if (ImGui::Button("Remove CameraComponent"))
            selected->RemoveComponent<CameraComponent>();
    }

    if (auto light = selected->GetComponent<LightComponent>())
    {
        ImGui::SeparatorText("Light Component");

        glm::vec3 color = light->GetColor();
        float intensity = light->GetIntensity();
        float radius = light->GetRadius();
        float falloff = light->GetFalloff();
        int type = light->GetType();

        if (ImGui::ColorEdit3("Color", &color.x))
            light->SetColor(color);
        if (ImGui::DragFloat("Intensity", &intensity, 0.05f, 0.0f, 10.0f))
            light->SetIntensity(intensity);
        if (ImGui::DragFloat("Radius", &radius, 1.0f, 10.0f, 1000.0f))
            light->SetRadius(radius);
        if (ImGui::DragFloat("Falloff", &falloff, 0.05f, 0.1f, 5.0f))
            light->SetFalloff(falloff);

        // const char* types[] = { "Point", "Directional" };
        // if (ImGui::Combo("Type", &type, types, IM_ARRAYSIZE(types)))
        //     light->SetType(type);

        if (ImGui::Button("Remove LightComponent"))
            selected->RemoveComponent<LightComponent>();
    }
    if (auto script = selected->GetComponent<ScriptComponent>())
    {
        ImGui::SeparatorText("Script Component");

        std::string path = script->GetScriptPath();
        ImGui::Text("Script Path:");
        ImGui::TextWrapped("%s", path.c_str());

        if (ImGui::Button("Load Script"))
        {
            std::string chosen = OpenFileDialog(FileDialogType::Scripts); // Add 'Scripts' filter if needed
            if (!chosen.empty())
                script->SetScriptPath(chosen);
        }

        ImGui::SameLine();
        if (ImGui::Button("Reload"))
        {
            script->ReloadScript();
        }

        if (ImGui::Button("Remove ScriptComponent"))
            selected->RemoveComponent<ScriptComponent>();
    }

    if (auto tilemap = selected->GetComponent<TilemapComponent>())
    {
        ImGui::SeparatorText("Tilemap Component");

        ImGui::Text("Refer to Tilemap Editor Window");

        if (ImGui::Button("Remove TilemapComponent"))
            selected->RemoveComponent<TilemapComponent>();
    }

    ImGui::End();

    if (auto tilemap = selected->GetComponent<TilemapComponent>())
    {
        tilemap->DrawEditorUI();
    }
}

void Engine::Run()
{
    while (!glfwWindowShouldClose(window))
    {
        glfwPollEvents();

        ImGui_ImplOpenGL3_NewFrame();
        ImGui_ImplGlfw_NewFrame();
        ImGui::NewFrame();
        ImGui::DockSpaceOverViewport(ImGui::GetMainViewport()->ID);

        float currentTime = glfwGetTime();
        static float lastTime = currentTime;
        float deltaTime = currentTime - lastTime;
        lastTime = currentTime;

        ShowDebugOverlay(deltaTime);
        Logger::Draw();

        if (ImGui::BeginMainMenuBar())
        {
            // — left‑aligned items —
            if (ImGui::Button(playing ? "Stop" : "Play"))
            {
                if (!playing)
                {
                    Logger::LogVerbose("[RestoreScene] Saving original scene");
                    SaveScene(tempScenePath);
                }
                else
                {
                    Logger::LogVerbose("[RestoreScene] Reloading original scene");
                    LoadScene(tempScenePath);
                }
                selected = nullptr;
                playing = !playing;
            }

            if (ImGui::BeginMenu("File"))
            {
                if (ImGui::MenuItem("Save Scene"))
                {
                    std::string file = OpenFileDialog(FileDialogType::Scenes);
                    if (!file.empty())
                        SaveScene(file);
                    selected = nullptr;
                }
                if (ImGui::MenuItem("Load Scene"))
                {
                    std::string file = OpenFileDialog(FileDialogType::Scenes);
                    Logger::LogInfo("[LoadScene] Loading Scene.");
                    if (!file.empty())
                        LoadScene(file);
                    selected = nullptr;
                }
                ImGui::EndMenu();
            }

            if (ImGui::BeginMenu("Options"))
            {
                // simple checkbox never closes the menu when you click it
                ImGui::Checkbox("Enable Lighting", &g_engineConfig.lighting_enabled);
                ImGui::Checkbox("Enable Gizmos", &g_engineConfig.settings.draw_gizmos);

                ImGui::EndMenu();
            }

            // — right‑aligned version text —
            const char *version_fmt = "Create Engine v%s";
            // build the text
            char buf[64];
            std::snprintf(buf, sizeof(buf), version_fmt, g_engineConfig.version.c_str());

            // calculate width of the text
            float textWidth = ImGui::CalcTextSize(buf).x;
            // move cursor to far right minus padding
            float avail = ImGui::GetWindowSize().x - ImGui::GetCursorPosX();
            ImGuiStyle &style = ImGui::GetStyle();
            ImGui::SameLine(avail - textWidth - style.FramePadding.x);

            ImGui::TextUnformatted(buf);

            ImGui::EndMainMenuBar();
        }

        ImGui::Begin("Scene Tree");

        if (ImGui::BeginPopupContextWindow("SceneTreeContext", ImGuiPopupFlags_MouseButtonRight))
        {
            if (ImGui::MenuItem("Create New"))
            {
                auto obj = std::make_shared<Object>("NewObject");
                objects.push_back(obj);
                selected = obj;
                ImGui::OpenPopup("RenameObject");
            }
            ImGui::Separator();
            if (ImGui::MenuItem("Create New Sprite"))
            {
                auto obj = std::make_shared<Object>("NewSprite");
                obj->AddComponent<SpriteComponent>();
                objects.push_back(obj);
                selected = obj;
                ImGui::OpenPopup("RenameObject");
            }
            if (ImGui::MenuItem("Create New Light"))
            {
                auto obj = std::make_shared<Object>("NewSprite");
                obj->AddComponent<LightComponent>();
                objects.push_back(obj);
                selected = obj;
                ImGui::OpenPopup("RenameObject");
            }
            ImGui::EndPopup();
        }

        for (auto &obj : objects)
            if (!obj->GetParent()) // Only draw root nodes
                DrawObjectNode(obj);

        ImGui::End();

        DrawInspectorUI(selected);

        // Viewport
        ImGui::Begin("Viewport", nullptr, ImGuiWindowFlags_NoScrollbar | ImGuiWindowFlags_NoScrollWithMouse);
        ImVec2 size = ImGui::GetContentRegionAvail();

        if (!playing && ImGui::IsWindowHovered())
        {
            if (ImGui::IsMouseDragging(ImGuiMouseButton_Right))
            {
                ImVec2 mouseDelta = ImGui::GetMouseDragDelta(ImGuiMouseButton_Right);
                ImGui::ResetMouseDragDelta(ImGuiMouseButton_Right);
                cameraPos -= glm::vec2(mouseDelta.x, mouseDelta.y) / cameraZoom;
            }

            float wheel = ImGui::GetIO().MouseWheel;
            if (wheel != 0.0f)
            {
                float zoomFactor = (wheel > 0) ? 1.1f : 1.0f / 1.1f;

                // Get mouse position in screen space
                ImVec2 mousePos = ImGui::GetMousePos();
                ImVec2 windowPos = ImGui::GetWindowPos();
                ImVec2 contentRegionMin = ImGui::GetWindowContentRegionMin();
                glm::vec2 relativeMouse = glm::vec2(mousePos.x - windowPos.x - contentRegionMin.x,
                                                    mousePos.y - windowPos.y - contentRegionMin.y);

                // Convert to world position before zoom
                glm::vec2 beforeZoom = (relativeMouse - glm::vec2(size.x / 2, size.y / 2)) / cameraZoom + cameraPos;

                // Apply zoom
                cameraZoom *= zoomFactor;
                cameraZoom = std::clamp(cameraZoom, 0.1f, 10.0f);

                // Convert to world position after zoom
                glm::vec2 afterZoom = (relativeMouse - glm::vec2(size.x / 2, size.y / 2)) / cameraZoom + cameraPos;

                // Offset camera so zoom is centered at mouse
                cameraPos += beforeZoom - afterZoom;
            }
        }

        Renderer::Resize((int)size.x, (int)size.y);
        Renderer::Begin();

        std::vector<std::shared_ptr<Object>> toDraw;
        std::vector<ScriptComponent *> scriptUpdates;

        toDraw.reserve(m_Reserved_draws);
        m_Reserved_draws = 0; // Reset

        Renderer::DrawEditorGrid(cameraPos, cameraZoom);

        // clear arrays each frame…
        toDraw.clear();
        scriptUpdates.clear();
        m_Reserved_draws = 0;

        // recursive collector now skips invisible objects (and their children)
        std::function<void(const std::shared_ptr<Object> &)> collect =
            [&](const std::shared_ptr<Object> &obj)
        {
            // if this object isn’t visible, skip it and its children
            if (!obj->GetVisable())
                return;

            // still visible → collect for drawing
            toDraw.push_back(obj);
            m_Reserved_draws += 1;

            // Collect lights
            if (auto light = obj->GetComponent<LightComponent>())
            {
                glm::vec2 world = obj->GetWorldPosition();
                glm::vec2 screen = (world - cameraPos) * cameraZoom + glm::vec2(Renderer::GetSize().x * 0.5f,
                                                                                Renderer::GetSize().y * 0.5f);
                Renderer::AddLight(screen,
                                   light->GetColor(),
                                   light->GetIntensity(),
                                   light->GetRadius() * cameraZoom);

                Renderer::DrawGizmoCircle(
                    world,
                    light->GetRadius(),
                    64,                    // segments
                    light->GetColor(), // circle color
                    cameraPos,
                    cameraZoom);
            }

            // Collect scripts
            if (playing)
            {
                if (auto script = obj->GetComponent<ScriptComponent>())
                    scriptUpdates.push_back(script.get());
            }

            // recurse into children
            for (const auto &child : obj->GetChildren())
                collect(child);
        };

        // Traverse only root objects
        for (const auto &obj : objects)
            if (!obj->GetParent())
                collect(obj);

        // Sort drawables by layer then Y
        std::sort(toDraw.begin(), toDraw.end(), [](auto const &a, auto const &b)
                  {
    if (a->layer != b->layer)
        return a->layer < b->layer;
    return a->GetWorldPosition().y < b->GetWorldPosition().y; });

        // Run script updates
        for (auto *script : scriptUpdates)
            script->OnUpdate(deltaTime);

        // Draw all sprites
        for (const auto &obj : toDraw)
        {
            if (auto sprite = obj->GetComponent<SpriteComponent>())
            {
                glm::vec2 worldPos = obj->GetWorldPosition();
                Renderer::DrawSprite(sprite.get(), worldPos, cameraZoom, cameraPos);
            }
        }

        Renderer::End();
        GLuint texID = Renderer::GetRenderTexture();
        ImGui::Image((ImTextureID)(uintptr_t)texID, size, ImVec2(0, 1), ImVec2(1, 0));
        ImGui::End();

        if (ImGui::BeginPopup("RenameObject"))
        {
            static char nameBuffer[128];
            static bool once = true;

            if (once && selected)
            {
                strcpy(nameBuffer, selected->GetName().c_str());
                once = false;
            }

            ImGui::InputText("##rename", nameBuffer, sizeof(nameBuffer));
            if (ImGui::IsKeyPressed(ImGuiKey_Enter) || ImGui::Button("OK"))
            {
                if (selected)
                    selected->SetName(nameBuffer);
                ImGui::CloseCurrentPopup();
                once = true;
            }

            ImGui::SameLine();
            if (ImGui::Button("Cancel"))
            {
                ImGui::CloseCurrentPopup();
                once = true;
            }

            ImGui::EndPopup();
        }

        // ImGui render
        ImGui::Render();
        int w, h;
        glfwGetFramebufferSize(window, &w, &h);
        glViewport(0, 0, w, h);
        glClear(GL_COLOR_BUFFER_BIT);
        ImGui_ImplOpenGL3_RenderDrawData(ImGui::GetDrawData());
        glfwSwapBuffers(window);

        for (auto &obj : pendingDeletion)
        {
            if (obj->GetParent())
            {
                obj->GetParent()->RemoveChild(obj.get());
            }
            else
            {
                objects.erase(std::remove_if(objects.begin(), objects.end(),
                                             [&](const std::shared_ptr<Object> &o)
                                             { return o == obj; }),
                              objects.end());
            }
        }
        pendingDeletion.clear();
    }
}

void Engine::DrawObjectNode(const std::shared_ptr<Object> &obj)
{
    ImGuiTreeNodeFlags flags = ImGuiTreeNodeFlags_OpenOnArrow |
                               ImGuiTreeNodeFlags_SpanAvailWidth |
                               (obj == selected ? ImGuiTreeNodeFlags_Selected : 0);

    bool open = ImGui::TreeNodeEx((void *)(intptr_t)obj->uid.id, flags, "%s", obj->GetName().c_str());

    if (ImGui::IsItemClicked())
        selected = obj;

    // Global or class-level list

    if (ImGui::BeginPopupContextItem())
    {
        if (ImGui::MenuItem("Rename"))
        {
            selected = obj;
            ImGui::OpenPopup("RenameObject");
        }
        if (ImGui::MenuItem("Delete"))
        {
            pendingDeletion.push_back(obj);
            ImGui::EndPopup();
            return; // Still return so you don't draw the item
        }
        if (ImGui::MenuItem("Create Child"))
        {
            auto child = std::make_shared<Object>("NewObject");
            obj->AddChild(child);
            selected = child;
            child->SetName("NewObject");
            ImGui::OpenPopup("RenameObject");
        }
        ImGui::EndPopup();
    }

    // === Drag and Drop ===
    if (ImGui::BeginDragDropSource())
    {
        ImGui::SetDragDropPayload("OBJECT", &obj, sizeof(obj));
        ImGui::Text("Move: %s", obj->GetName().c_str());
        ImGui::EndDragDropSource();
    }

    if (ImGui::BeginDragDropTarget())
    {
        if (const ImGuiPayload *payload = ImGui::AcceptDragDropPayload("OBJECT"))
        {
            auto dragged = *(std::shared_ptr<Object> *)payload->Data;
            if (dragged != obj)
                obj->AddChild(dragged);
        }
        ImGui::EndDragDropTarget();
    }

    // === Children ===
    if (open)
    {
        for (auto &child : obj->GetChildren())
            DrawObjectNode(child);
        ImGui::TreePop();
    }
}

void Engine::SaveScene(const std::string &path)
{
    YAML::Emitter out;
    YAML::Emitter sceneData;

    // Serialize object list only
    sceneData << YAML::BeginSeq;
    for (const auto &obj : objects)
        obj->Save(sceneData);
    sceneData << YAML::EndSeq;

    // Hash the serialized object data
    std::string objectData = sceneData.c_str();
    unsigned char hash[SHA256_DIGEST_LENGTH];
    SHA256(reinterpret_cast<const unsigned char *>(objectData.c_str()), objectData.size(), hash);

    std::ostringstream hashHex;
    for (int i = 0; i < SHA256_DIGEST_LENGTH; ++i)
        hashHex << std::hex << std::setw(2) << std::setfill('0') << static_cast<int>(hash[i]);

    // Build final full YAML output
    out << YAML::BeginMap;
    out << YAML::Key << "engine_version" << YAML::Value << g_engineConfig.version;
    out << YAML::Key << "scene_name" << YAML::Value << std::filesystem::path(path).stem().string();
    out << YAML::Key << "scene_hash" << YAML::Value << hashHex.str();
    out << YAML::Key << "format_version" << YAML::Value << 1;
    out << YAML::Key << "objects" << YAML::Value << YAML::Load(objectData);
    out << YAML::EndMap;

    std::ofstream file(path);
    file << out.c_str();
}

bool VerifySceneHash(const YAML::Node &root)
{
    if (!root["scene_hash"] || !root["objects"])
        return false;

    YAML::Emitter sceneOnly;
    sceneOnly << root["objects"];

    std::string sceneString = sceneOnly.c_str();
    unsigned char hash[SHA256_DIGEST_LENGTH];
    SHA256(reinterpret_cast<const unsigned char *>(sceneString.c_str()), sceneString.size(), hash);

    std::ostringstream hashHex;
    for (int i = 0; i < SHA256_DIGEST_LENGTH; ++i)
        hashHex << std::hex << std::setw(2) << std::setfill('0') << static_cast<int>(hash[i]);

    return hashHex.str() == root["scene_hash"].as<std::string>();
}

void Engine::LoadScene(const std::string &path)
{
    Logger::LogDebug("[LoadScene] Reading Scene File.");

    YAML::Node root = YAML::LoadFile(path);

    Logger::LogDebug("[LoadScene] Verifying Scene");

    if (!root["engine_version"] || !root["format_version"] || !root["scene_name"])
    {
        Logger::LogError("[LoadScene] Missing required metadata!");
        return;
    }

    if (root["engine_version"].as<std::string>() != "0.1.0")
    {
        Logger::LogError("[LoadScene] Version mismatch! Expected 0.1.0, got %s", root["engine_version"].as<std::string>().c_str());
        return;
    }

    if (!VerifySceneHash(root))
    {
        Logger::LogWarning("[LoadScene] Scene hash does not match! File may be corrupted or tampered.");
    }

    Logger::LogDebug("[LoadScene] Reseting Scene.");

    objects.clear();

    Logger::LogDebug("[LoadScene] Recreting Objects");

    const auto &objectArray = root["objects"];
    for (const auto &node : objectArray)
    {
        auto obj = std::make_shared<Object>("[DefaultObject]");
        obj->Load(node);
        objects.push_back(obj);
    }

    Logger::LogInfo("[LoadScene] Loaded scene: %s", root["scene_name"].as<std::string>().c_str());
}

void Engine::Shutdown()
{
    ImGui_ImplOpenGL3_Shutdown();
    ImGui_ImplGlfw_Shutdown();
    ImGui::DestroyContext();
    glfwDestroyWindow(window);
    glfwTerminate();

    std::filesystem::remove(tempScenePath);
}
