#include "Object.h"
#include "../Components/Component.h"
#include "../Components/SpriteComponent.h"
#include "../Components/CameraComponent.h"
#include "../Components/LightComponent.h"
#include "../Components/TilemapComponent.h"
#include "../Components/TextComponent.h"
#include "../Components/ScriptComponent.h"

#include "../utils/Logging.h"
#include "../utils/UID.h"

#include <algorithm>

Object::Object(const std::string &name)
    : name(name), localPosition(0.0f, 0.0f), uid() {}

Object::~Object() {}

void Object::SetParent(Object *newParent)
{
    if (parent)
        parent->RemoveChild(this);
    parent = newParent;
}

void Object::AddChild(std::shared_ptr<Object> child)
{
    child->SetParent(this);
    children.push_back(child);
}

void Object::RemoveChild(Object *child)
{
    children.erase(std::remove_if(children.begin(), children.end(),
                                  [child](const std::shared_ptr<Object> &obj)
                                  {
                                      return obj.get() == child;
                                  }),
                   children.end());
}

glm::vec2 Object::GetLocalPosition() const
{
    return localPosition;
}

bool Object::GetVisable() const
{
    return visable;
}

void Object::SetVisable(bool state)
{
    visable = state;
}


void Object::SetLocalPosition(glm::vec2 pos)
{
    localPosition = pos;
}

glm::vec2 Object::GetWorldPosition() const
{
    if (parent)
        return parent->GetWorldPosition() + localPosition;
    return localPosition;
}

const std::string &Object::GetName() const { return name; }
void Object::SetName(const std::string &n) { name = n; }
std::vector<std::shared_ptr<Object>> &Object::GetChildren() { return children; }
Object *Object::GetParent() const { return parent; }

void Object::Save(YAML::Emitter &out) const
{
    Logger::LogVerbose("[LoadScene] Saving Object: [%s, %d]", name.c_str(), uid.id);


    out << YAML::BeginMap;
    out << YAML::Key << "name" << YAML::Value << name;
    out << YAML::Key << "uid" << YAML::Value << uid.uuid;
    out << YAML::Key << "id" << YAML::Value << uid.id;
    out << YAML::Key << "position" << YAML::Value << YAML::Flow << YAML::BeginSeq << localPosition.x << localPosition.y << YAML::EndSeq;
    out << YAML::Key << "layer" << YAML::Value << layer;
    out << YAML::Key << "visable" << YAML::Value << visable;


    out << YAML::Key << "components" << YAML::Value << YAML::BeginSeq;
    for (const auto &comp : components)
    {
        Logger::LogVerbose("[LoadScene] Saving Component: %s", comp->GetName().c_str());
        comp->Save(out);
    }
    out << YAML::EndSeq;

    out << YAML::Key << "children" << YAML::Value << YAML::BeginSeq;
    for (const auto &child : children)
        child->Save(out);
    out << YAML::EndSeq;

    out << YAML::EndMap;
}

void Object::Load(const YAML::Node &node)
{
    name = node["name"].as<std::string>();

    if (node["uid"])
        uid.uuid = node["uid"].as<std::string>();
    else
        uid.uuid = GenerateUUID();

    if (node["id"])
        uid.id = node["id"].as<int>();

    auto pos = node["position"];
    if (pos && pos.IsSequence() && pos.size() == 2)
    {
        localPosition.x = pos[0].as<float>();
        localPosition.y = pos[1].as<float>();
    }

    if (node["layer"])
        layer = node["layer"].as<int>();
        
    if (node["visable"])
        visable = node["visable"].as<bool>();
        

    Logger::LogVerbose("[LoadScene] Loading Object: [%s, %d]", name.c_str(), uid.id);

    if (node["components"])
    {
        for (const auto &compNode : node["components"])
        {
            std::string type = compNode["type"].as<std::string>();
            Logger::LogVerbose("[LoadScene] Createing Component: %s", type.c_str());

            if (type == "SpriteComponent")
            {
                auto comp = AddComponent<SpriteComponent>();
                comp->Load(compNode);
            }
            else if (type == "CameraComponent")
            {
                auto comp = AddComponent<CameraComponent>();
                comp->Load(compNode);
            }
            else if (type == "LightComponent")
            {
                auto comp = AddComponent<LightComponent>();
                comp->Load(compNode);
            }
            else if (type == "TilemapComponent")
            {
                auto comp = AddComponent<TilemapComponent>();
                comp->Load(compNode);
            }
            else if (type == "TextComponent")
            {
                auto comp = AddComponent<TextComponent>();
                comp->Load(compNode);
            }
            else if (type == "ScriptComponent")
            {
                auto comp = AddComponent<ScriptComponent>();
                comp->Load(compNode);
            }
        }
    }

    if (node["children"])
    {
        for (const auto &childNode : node["children"])
        {
            auto child = std::make_shared<Object>("Child");
            child->Load(childNode);
            AddChild(child);
        }
    }
}
