#include "Logging.h"
#include <imgui.h>
#include <cstdarg>
#include <cstdio>
#include <iostream>

std::vector<Logger::LogMessage> Logger::s_Messages;
bool Logger::s_AutoScroll = true;
bool Logger::s_ShowInfo = true;
bool Logger::s_ShowWarning = true;
bool Logger::s_ShowError = true;
bool Logger::s_ShowDebug = false;
bool Logger::s_ShowVerbose = false;

const char *Logger::ToString(Level level)
{
    switch (level)
    {
    case Info:
        return "Info";
    case Warning:
        return "Warning";
    case Error:
        return "Error";
    case Debug:
        return "Debug";
    case Verbose:
        return "Verbose";
    default:
        return "Unknown";
    }
}

const char *Logger::GetAnsiColor(Level level)
{
    switch (level)
    {
    case Info:
        return "\033[0;37m"; // Gray
    case Warning:
        return "\033[1;33m"; // Yellow
    case Error:
        return "\033[1;31m"; // Red
    case Debug:
        return "\033[1;36m"; // Cyan
    case Verbose:
        return "\033[0;90m"; // Dim gray

    default:
        return "\033[0m"; // Reset
    }
}

void Logger::Log(Level level, const char *fmt, ...)
{
    va_list args;
    va_start(args, fmt);
    LogVA(level, fmt, args);
    va_end(args);
}

void Logger::LogVA(Level level, const char *fmt, va_list args)
{
    char buffer[1024];
    vsnprintf(buffer, sizeof(buffer), fmt, args);
    s_Messages.push_back({buffer, level});

    std::cout << GetAnsiColor(level)
              << "[Logger][" << ToString(level) << "] "
              << buffer << "\033[0m" << std::endl;
}

void Logger::LogInfo(const char *fmt, ...)
{
    va_list args;
    va_start(args, fmt);
    LogVA(Info, fmt, args);
    va_end(args);
}

void Logger::LogWarning(const char *fmt, ...)
{
    va_list args;
    va_start(args, fmt);
    LogVA(Warning, fmt, args);
    va_end(args);
}

void Logger::LogError(const char *fmt, ...)
{
    va_list args;
    va_start(args, fmt);
    LogVA(Error, fmt, args);
    va_end(args);
}

void Logger::LogDebug(const char *fmt, ...)
{
    va_list args;
    va_start(args, fmt);
    LogVA(Debug, fmt, args);
    va_end(args);
}

void Logger::LogVerbose(const char *fmt, ...)
{
    va_list args;
    va_start(args, fmt);
    LogVA(Verbose, fmt, args);
    va_end(args);
}

void Logger::Clear()
{
    s_Messages.clear();
}

void Logger::Draw(const char *title)
{
    ImGui::Begin(title);

    if (ImGui::Button("Clear"))
        Clear();
    ImGui::SameLine();
    ImGui::Checkbox("Auto-scroll", &s_AutoScroll);
    ImGui::Separator();

    ImGui::Checkbox("Info", &s_ShowInfo);
    ImGui::SameLine();
    ImGui::Checkbox("Warning", &s_ShowWarning);
    ImGui::SameLine();
    ImGui::Checkbox("Error", &s_ShowError);
    ImGui::SameLine();
    ImGui::Checkbox("Debug", &s_ShowDebug);
    ImGui::SameLine();
    ImGui::Checkbox("Verbose", &s_ShowVerbose);

    ImGui::Separator();

    ImGui::BeginChild("LogMessages");

    for (const auto &msg : s_Messages)
    {
        if ((msg.level == Info && !s_ShowInfo) ||
            (msg.level == Warning && !s_ShowWarning) ||
            (msg.level == Error && !s_ShowError) ||
            (msg.level == Debug && !s_ShowDebug) ||
            (msg.level == Verbose && !s_ShowVerbose))
        {
            continue;
        }

        ImVec4 color;
        switch (msg.level)
        {
        case Info:
            color = ImVec4(0.8f, 0.8f, 0.8f, 1.0f);
            break; // Light gray
        case Warning:
            color = ImVec4(1.0f, 0.85f, 0.3f, 1.0f);
            break; // Yellow
        case Error:
            color = ImVec4(1.0f, 0.3f, 0.3f, 1.0f);
            break; // Red
        case Debug:
            color = ImVec4(0.3f, 0.9f, 1.0f, 1.0f);
            break; // Cyan
        case Verbose:
            color = ImVec4(0.5f, 0.5f, 0.5f, 1.0f);
            break; // Dim gray
        default:
            color = ImVec4(1.0f, 1.0f, 1.0f, 1.0f);
            break; // Fallback white
        }

        ImGui::PushStyleColor(ImGuiCol_Text, color);
        ImGui::Text("[%s] %s", ToString(msg.level), msg.text.c_str());
        ImGui::PopStyleColor();
    }

    if (s_AutoScroll && ImGui::GetScrollY() >= ImGui::GetScrollMaxY())
        ImGui::SetScrollHereY(1.0f);

    ImGui::EndChild();
    ImGui::End();
}
